module Resources

  class AccountMapping < BaseResource
    include ThirdPartyPrompter
    include AccountMappingPrompter

    def list
      tp_id = third_party_id
      result = get("services/third_parties/#{tp_id}/account_mappings")
      puts "\nAccount Mappings for third party: #{tp_id}"
      table = Terminal::Table.new headings: [ 'Third Party', 'Account', 'Account Code' ] do |t|
      result.each do | account_mapping |
        t << [ "#{account_mapping['third_party_name']} (#{account_mapping['third_party_id']})",
               "#{account_mapping['account_name']} (#{account_mapping['account_id']})",
               account_mapping['account_code'] ]
        end
      end
      prompter.say table.to_s
      puts
    end

    def show
      tp_id = third_party_id
      am_id = account_mapping_id
      account_mapping = get("services/third_parties/#{tp_id}/account_mappings/#{am_id}")
      @am_third_party_id = account_mapping['third_party_id']
      @am_account_id = account_mapping['account_id']
      @am_account_code = account_mapping['account_code']
      puts "\nAccount Mapping for third party: #{tp_id}/#{am_id}"
      table = Terminal::Table.new headings: [ 'Third Party', 'Account', 'Account Code' ] do |t|
        t << [ "#{account_mapping['third_party_name']} (#{account_mapping['third_party_id']})",
               "#{account_mapping['account_name']} (#{account_mapping['account_id']})",
               account_mapping['account_code'] ]
      end
      puts table
      puts
    end

    def create
      tp_id = third_party_id
      params = { 
        account_mapping: {
          account_id: prompter.ask("\nabaqis Account ID: ", Integer) { |q| q.validate = lambda { | a | is_valid_object?('Account', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid account ID" },
          account_code: prompter.ask("\nThird party Account Code: ") { |q| q.validate = /\A.{1,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid third party account ID" }
        }
      }
      result = post("services/third_parties/#{tp_id}/account_mappings", params)
      puts "\nID of new account mapping: #{result['id']}"
      puts
    end

    def update
      tp_id = third_party_id
      am_id = account_mapping_id
      show
      params = { 
        account_mapping: {
          account_id: prompter.ask("\nabaqis Account ID: ", Integer) { |q| q.default = @am_account_id; q.validate = lambda { | a | is_valid_object?('Account', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid account ID" },
          account_code: prompter.ask("\nThird party Account Code: ")  { |q| q.default = @am_account_code; q.validate = /\A.{1,255}\z/; q.responses[:not_valid] = "\nNot a valid third party account ID" }
        }
      }
      result = put("services/third_parties/#{tp_id}/account_mappings/#{am_id}", params)
      puts "\nID of updated account mapping: #{result['id']}"
      puts
    end

    def destroy
      tp_id = third_party_id
      am_id = account_mapping_id
      show
      if prompter.agree("\nAre you sure you want to destroy this account mapping? (y/n) ", true)
        puts
        result = delete("services/third_parties/#{tp_id}/account_mappings/#{am_id}")
        puts "\nID of deleted account mapping: #{result['id']}"
      else
        puts "\nCancelled deletion"
      end
      puts
    end

  end

end
