module Resources

  class ConfiguredAccount < BaseResource
    include ThirdPartyPrompter
    include ServiceDefinitionPrompter
    include ConfiguredAccountPrompter

    def list
      tp_id = third_party_id
      sd_id = service_definition_id
      result = get("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_accounts")
      puts "\nConfigured accounts for third party: #{tp_id}, service definition: #{sd_id}"
      table = Terminal::Table.new headings: [ 'Service Def Id', 'Account ID', 'User name', 'Enabled', 'Password', 'Token' ] do |t|
      result.each do | configured_account |
        t << [ configured_account['service_definition_id'],
               "#{configured_account['account_name']} (#{configured_account['account_id']})",
               configured_account['username'],
               configured_account['enabled'],
               configured_account['password'],
               configured_account['token'] ]
        end
      end
      prompter.say table.to_s
      puts
    end

    def show
      tp_id = third_party_id
      sd_id = service_definition_id
      ca_id = configured_account_id
      configured_account = get("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_accounts/#{ca_id}")
      @ca_account_id = configured_account['account_id']
      @ca_username = configured_account['username']
      @ca_enabled = configured_account['enabled']
      @ca_password = configured_account['password']
      @ca_token = configured_account['token']
      puts "\nConfigured account for third party: #{tp_id}/#{options[:id]}"
      table = Terminal::Table.new headings: [ 'Service Def Id', 'Account ID', 'User name', 'Enabled', 'Password', 'Token' ] do |t|
        t << [ configured_account['service_definition_id'],
               "#{configured_account['account_name']} (#{configured_account['account_id']})",
               configured_account['username'],
               configured_account['enabled'],
               configured_account['password'],
               configured_account['token'] ]
      end
      puts table
      puts
    end

    def create
      tp_id = third_party_id
      sd_id = service_definition_id
      params = { 
        configured_account: {
          account_id: prompter.ask("\nAccount ID: ", Integer) { |q| q.validate = lambda { | a | is_valid_object?('Account', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Account ID" },
          username: prompter.ask("\nUser name: ") { |q| q.validate = /\A.{0,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nInvalid user name" },
          enabled: prompter.agree("\nEnabled? (y/n) ", true)  
        }
      }
      puts
      password = prompter.ask("\nPassword: ")
      token = prompter.ask("\nToken: ")
      if (password && password.size > 0) || (token && token.size > 0) 
        params[:credential] = {
        }
        params[:credential][:password] = password if password && password.size > 0
        params[:credential][:token] = token if token && token.size > 0
      end
      result = post("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_accounts", params)
      puts "\nID of new configured account: #{result['id']}"
      puts
    end

    def update
      clear_default = ->(field) { field == "^" ? '' : field }
      tp_id = third_party_id
      sd_id = service_definition_id
      ca_id = configured_account_id
      show
      params = { 
        configured_account: {
          account_id: prompter.ask("\nAccount ID: ", Integer) { |q| q.default = @ca_account_id; q.validate = lambda { | a | is_valid_object?('Account', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Account ID" },
          username: prompter.ask("\nUser name (^ to clear): ", clear_default) { |q| q.default = @ca_username; q.validate = /\A.{0,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nInvalid user name" },
          enabled: prompter.agree("\nEnabled? ", true)  { |q| q.default = @ca_enabled }
        }
      }
      puts
      password = prompter.ask("\nPassword (^ to clear): ", clear_default) { |q| q.default = @ca_password }
      token = prompter.ask("\nToken (^ to clear): ", clear_default) { |q| q.default = @ca_token }
      if (password && password.size > 0) || (token && token.size > 0) 
        params[:credential] = {
        }
        params[:credential][:password] = password if password && password.size > 0
        params[:credential][:token] = token if token && token.size > 0
      end
      result = put("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_accounts/#{ca_id}", params)
      puts "\nID of updated configured account: #{result['id']}"
      puts
    end

    def destroy
      tp_id = third_party_id
      sd_id = service_definition_id
      ca_id = configured_account_id
      show
      if prompter.agree("\nAre you sure you want to destroy this configured account? (y/n) ", true)
        puts
        result = delete("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_accounts/#{ca_id}")
        puts "\nID of deleted configured account: #{result['id']}"
      else
        puts "\nCancelled deletion"
      end
      puts
    end

  end

end
