module Resources

  class ConfiguredFacility < BaseResource
    include ThirdPartyPrompter
    include ServiceDefinitionPrompter
    include ConfiguredAccountPrompter
    include ConfiguredFacilityPrompter

    def list
      tp_id = third_party_id
      ca_id = configured_account_id
      sd_id = service_definition_id
      result = get("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_facilities?configured_account_id=#{ca_id}")
      puts "\nConfigured facilities for third party: #{tp_id}, service definition: #{sd_id}"
      table = Terminal::Table.new headings: [ 'Service Def Id', 'Facility ID', 'User name', 'Enabled', 'Password', 'Token' ] do |t|
      result.each do | configured_facility |
        t << [ configured_facility['service_definition_id'],
               "#{configured_facility['facility_name']} (#{configured_facility['facility_id']})",
               configured_facility['username'],
               configured_facility['enabled'],
               configured_facility['password'],
               configured_facility['token'] ]
        end
      end
      prompter.say table.to_s
      puts
    end

    def show
      tp_id = third_party_id
      sd_id = service_definition_id
      cf_id = configured_facility_id
      configured_facility = get("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_facilities/#{cf_id}")
      @cf_facility_id = configured_facility['facility_id']
      @cf_username = configured_facility['username']
      @cf_enabled = configured_facility['enabled']
      @cf_password = configured_facility['password']
      @cf_token = configured_facility['token']
      puts "\nConfigured facility for third party: #{tp_id}/#{options[:id]}"
      table = Terminal::Table.new headings: [ 'Service Def Id', 'Facility ID', 'User name', 'Enabled', 'Password', 'Token' ] do |t|
        t << [ configured_facility['service_definition_id'],
               "#{configured_facility['facility_name']} (#{configured_facility['facility_id']})",
               configured_facility['username'],
               configured_facility['enabled'],
               configured_facility['password'],
               configured_facility['token'] ]
      end
      puts table
      puts
    end

    def create
      tp_id = third_party_id
      sd_id = service_definition_id
      params = { 
        configured_facility: {
          facility_id: prompter.ask("\nFacility ID: ", Integer) { |q| q.validate = lambda { | f | is_valid_object?('Facility', f) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Facility ID" },
          username: prompter.ask("\nUser name: ") { |q| q.validate = /\A.{0,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nInvalid user name" },
          enabled: prompter.agree("\nEnabled? (y/n) ", true)  
        }
      }
      puts
      password = prompter.ask("\nPassword: ")
      token = prompter.ask("\nToken: ")
      if (password && password.size > 0) || (token && token.size > 0) 
        params[:credential] = {
        }
        params[:credential][:password] = password if password && password.size > 0
        params[:credential][:token] = token if token && token.size > 0
      end
      result = post("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_facilities", params)
      puts "\nID of new configured facility: #{result['id']}"
      puts
    end

    def update
      clear_default = ->(field) { field == "^" ? '' : field }
      tp_id = third_party_id
      sd_id = service_definition_id
      cf_id = configured_facility_id
      show
      params = { 
        configured_facility: {
          facility_id: prompter.ask("\nFacility ID: ", Integer) { |q| q.default = @cf_facility_id; q.validate = lambda { | f | is_valid_object?('Facility', f) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Facility ID" },
          username: prompter.ask("\nUser name (^ to clear): ", clear_default) { |q| q.default = @cf_username; q.validate = /\A.{0,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nInvalid user name" },
          enabled: prompter.agree("\nEnabled? ", true)  { |q| q.default = @cf_enabled }
        }
      }
      puts
      password = prompter.ask("\nPassword (^ to clear): ", clear_default) { |q| q.default = @cf_password }
      token = prompter.ask("\nToken (^ to clear): ", clear_default) { |q| q.default = @cf_token }
      if (password && password.size > 0) || (token && token.size > 0) 
        params[:credential] = {
        }
        params[:credential][:password] = password if password && password.size > 0
        params[:credential][:token] = token if token && token.size > 0
      end
      result = put("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_facilities/#{cf_id}", params)
      puts "\nID of updated configured facility: #{result['id']}"
      puts
    end

    def destroy
      tp_id = third_party_id
      sd_id = service_definition_id
      cf_id = configured_facility_id
      show
      if prompter.agree("\nAre you sure you want to destroy this configured facility? (y/n) ", true)
        puts
        result = delete("services/third_parties/#{tp_id}/service_definitions/#{sd_id}/configured_facilities/#{cf_id}")
        puts "\nID of deleted configured facility: #{result['id']}"
      else
        puts "\nCancelled deletion"
      end
      puts
    end

  end

end
