module Resources

  class FacilityMapping < BaseResource
    include ThirdPartyPrompter
    include ConfiguredAccountPrompter
    include ServiceDefinitionPrompter
    include FacilityMappingPrompter

    def list
      tp_id = third_party_id
      result = get("services/third_parties/#{tp_id}/facility_mappings") 
      puts "\nFacility Mappings for third party: #{tp_id}"
      table = Terminal::Table.new headings: [ 'Third Party Id', 'Facility Id', 'Facility Code', 'Top Level' ] do |t|
      result.each do | facility_mapping |
        t << [ "#{facility_mapping['third_party_name']} (#{facility_mapping['third_party_id']})",
               "#{facility_mapping['facility_name']} (#{facility_mapping['facility_id']})",
               facility_mapping['facility_code'],
               facility_mapping['top_level'] ]
        end
      end
      prompter.say table.to_s
      puts
    end

    def show
      tp_id = third_party_id
      fm_id = facility_mapping_id
      facility_mapping = get("services/third_parties/#{tp_id}/facility_mappings/#{fm_id}")
      @fm_third_party_id = facility_mapping['third_party_id']
      @fm_facility_id = facility_mapping['facility_id']
      @fm_facility_code = facility_mapping['facility_code']
      @fm_top_level = facility_mapping['top_level']
      puts "\nFacility Mapping for third party: #{tp_id}/#{fm_id}"
      table = Terminal::Table.new headings: [ 'Third Party Id', 'Facility Id', 'Facility Code', 'Top Level' ] do |t|
        t << [ "#{facility_mapping['third_party_name']} (#{facility_mapping['third_party_id']})",
               "#{facility_mapping['facility_name']} (#{facility_mapping['facility_id']})",
               facility_mapping['facility_code'],
               facility_mapping['top_level'] ]
      end
      puts table
      puts
    end

    def create
      tp_id = third_party_id
      params = { 
        facility_mapping: {
          facility_id: prompter.ask("\nabaqis Facility ID: ", Integer) { |q| q.validate = lambda { | a | is_valid_object?('Facility', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Facility ID" },
          facility_code: prompter.ask("\nThird party facility id: ") { |q| q.validate = /\A.{1,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid third party facility ID" },
          top_level: prompter.agree("\nTop Level? ", true)
        }
      }
      puts
      result = post("services/third_parties/#{tp_id}/facility_mappings", params)
      puts "\nID of new facility mapping: #{result['id']}"
      puts
    end

    def update
      tp_id = third_party_id
      fm_id = facility_mapping_id
      show
      params = { 
        facility_mapping: {
          facility_id: prompter.ask("\nabaqis Facility ID: ", Integer) { |q| q.default = @fm_facility_id; q.validate = lambda { | a | is_valid_object?('Facility', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Facility ID" },
          facility_code: prompter.ask("\nThird party facility id: ")  { |q| q.default = @fm_facility_code; q.validate = /\A.{1,255}\z/; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid third party facility ID" },
          top_level: prompter.agree("\nTop level? ", true) { |q| q.default = @fm_top_level }
        }
      }
      puts
      result = put("services/third_parties/#{tp_id}/facility_mappings/#{fm_id}", params)
      puts "\nID of updated facility mapping: #{result['id']}"
      puts
    end

    def destroy
      tp_id = third_party_id
      fm_id = facility_mapping_id
      show
      if prompter.agree("\nAre you sure you want to destroy this facility mapping? (y/n) ", true)
        puts
        result = delete("services/third_parties/#{tp_id}/facility_mappings/#{fm_id}")
        puts "\nID of deleted facility mapping: #{result['id']}"
      else
        puts "\nCancelled deletion"
      end
      puts
    end

  end

end
