module Resources

  class SltcRegistration < BaseResource
    include ThirdPartyPrompter
    include ServiceDefinitionPrompter
    include SltcRegistrationPrompter
    
    def list
      status = prompter.choose do | menu |
        menu.prompt = "\nSelect the status to filter with: "

        menu.choice('All', 'Specify no status') { -1 }
        menu.choice('New', 'New registrations') { 0 }
        menu.choice('Completed', 'Completed registrations') { 1 }
      end
      
      filter = status >= 0 ? "?status=#{status}" : ''
      result = get("services/sltc_registrations#{filter}")

      puts "\nSimpleLTC Registrations"
      table = Terminal::Table.new headings: [ 'Status', 'Company Name', 'Initiated By', 'Created At' ] do |t|
        result.each do | registration |
          t << [ registration['status'],
                 registration['company_name'],
                 registration['initiated_by'],
                 registration['created_at'] ]
        end
      end
      prompter.say table.to_s
      puts
    end
    
    def complete
      tp_id = third_party_id
      sd_id = service_definition_id
      reg_id = sltc_registration_id

      params = {
        third_party_id: tp_id,
        service_definition_id: sd_id,
        account_id: prompter.ask("\nAccount ID: ", Integer) { |q| q.validate = lambda { | a | is_valid_object?('Account', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Account ID" },
        baseline_days: prompter.ask("\nNumber of days for baseline: ", Integer) { |q| q.default = 180; q.responses[:not_valid] = "\nNot a valid number of days" }
      }
      result = post("services/sltc_registrations/#{reg_id}/complete", params)
      puts result.inspect
      puts
    end

    def configure
      tp_id = third_party_id
      sd_id = service_definition_id
      reg_id = sltc_registration_id

      params = {
        third_party_id: tp_id,
        service_definition_id: sd_id,
        account_id: prompter.ask("\nAccount ID: ", Integer) { |q| q.validate = lambda { | a | is_valid_object?('Account', a) }; q.responses[:ask_on_error] = :question; q.responses[:not_valid] = "\nNot a valid Account ID" },
        configure_only: 'true'
      }
      result = post("services/sltc_registrations/#{reg_id}/complete", params)
      puts result.inspect
      puts
    end

  end

end
